/**
* This adds a Flood Fill function to a BitmapData object.
* 
* It will perform an iterative fill from the given x/y coordinate, at the given color.
* 
* If you specify the tolerance you can allow it to fill into semi-transparent areas.
*
* It will ignore fills on the color black, or onto the same color as the fill color given.
* 
* @param {integer} x - The x coordinate to begin the fill from.
* @param {integer} y - The y coordinate to begin the fill from.
* @param {integer} [red=255] - The red component of the fill color. A value between 0 and 255.
* @param {integer} [green=0] - The green component of the fill color. A value between 0 and 255.
* @param {integer} [blue=0] - The blue component of the fill color. A value between 0 and 255.
* @param {integer} [alpha=255] - The alpha component of the fill color. A value between 0 and 255.
* @param {integer} [tolerance=0] - The tolerance applied to the fill search. A value between 0 (no tolerance) and 255.
* @param {boolean} [ignoreBlack=true] - By default the fill routine will ignore black pixels, allowing you to flood-fill line art.
* @return {boolean} True if the flood fill routine ran, otherwise false.
*/
Phaser.BitmapData.prototype.floodFill = function(x, y, color, data, copyPixels, redraw)
{
    let above = 0, below = 0, b = 0, c = false, e = 0, n = 0;

    let b32 = this.pixels;

    let i = x + y * this.width;
    let t = b32.length - this.width;

    let target = b32[i];

    if(target === color) return false;

    let queue = [i];

    while(queue.length)
    {
        i = queue.pop();
        y = (i / this.width) >> 0;
        b = y * this.width;
        e = b + this.width;

        while(i >= b && data[i * 4 + 3] == 0) i--;

        i++;
        above = 0;
        below = 0;

        while(i < e && data[i * 4 + 3] == 0)
        {            
            b32[i] = color;
            copyPixels[i] = 0;

            if(i >= this.width)
            {
                n = i - this.width;
                c = (b32[n] === color) ? false : data[n * 4 + 3] == 0;

                if(!above && c)
                {
                    queue.push(n);
                    above = 1;
                }
                else if(above && !c) above = 0;
            }

            if(i < t)
            {
                n = i + this.width;
                c = (b32[n] === color) ? false : data[n * 4 + 3] == 0;

                if(!below && c)
                {
                    queue.push(n);
                    below = 1;
                }
                else if(below && !c) below = 0;
            }

            i++;
        }
    }

    if(redraw) this.redraw();

    return true;
};

//
Phaser.BitmapData.prototype.redraw = function()
{
    this.ctx.putImageData(this.imageData, 0, 0);
    this.update();
};